const util = require('util')
const OAuth2Strategy = require('passport-oauth2')
const InternalOAuthError = require('passport-oauth2').InternalOAuthError

function Strategy (options, verify) {
  options = options || {}

  options.authorizationURL =
    options.authorizationURL ||
    'https://idecisiongames.com/oauth2/dialog/authorize'
  options.tokenURL =
    options.tokenURL || 'https://idecisiongames.com/oauth2/oauth/token'
  options.scope = options.scope || [] // ['email', 'profile']

  OAuth2Strategy.call(this, options, verify)

  this.options = options
  this.name = 'idecisiongames'
  this.profileUrl =
    options.profileUrl || 'https://idecisiongames.com/oauth2/api/userinfo'
}

util.inherits(Strategy, OAuth2Strategy)

Strategy.prototype.userProfile = function (accessToken, done) {
  let self = this
  this._oauth2.get(this.profileUrl, accessToken, function (err, body, res) {
    if (err) {
      return done(new InternalOAuthError('failed to fetch user profile', err))
    }

    let json, profile

    if ('string' === typeof body) {
      try {
        json = JSON.parse(body)
      } catch (e) {
        return done(e)
      }
    } else if ('object' === typeof body) {
      json = body
    }

    profile = parseProfile(json)
    profile.provider = self.name
    profile._raw = body
    profile._json = json

    done(null, profile)
  })
}

function parseProfile (data) {
  data = JSON.parse(JSON.stringify(data))

  if (data.avatar === '/img/a/user.png') {
    delete data.avatar
  }
  return data
}

// Expose constructor.
module.exports = Strategy;